@@ -9,12 +9,19 @@ from SLOPpy.subroutines.math_functions import *
 from SLOPpy.subroutines.bayesian_emcee import *
 # from SLOPpy.subroutines.rebin_subroutines import *
 
-__all__ = ['compute_transmission_mcmc']
+__all__ = ['compute_transmission_mcmc', 'compute_transmission_mcmc_iterative']
 
 subroutine_name = 'transmission_mcmc'
 
 
-def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
+
+def compute_transmission_mcmc_iterative(config_in, lines_label):
+
+    pca_parameters = from_config_get_pca_parameters(config_in)
+    for it in range(0, pca_parameters.get('iterations',5)):
+        compute_transmission_mcmc(config_in, lines_label, reference='planetRF', pca_iteration=it)
+
+def compute_transmission_mcmc(config_in, lines_label, reference='planetRF', pca_iteration=-1):
 
     night_dict = from_config_get_nights(config_in)
     planet_dict = from_config_get_planet(config_in)
@@ -26,9 +33,18 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
     """
 
     spectral_lines = from_config_get_spectral_lines(config_in)
-    line_iter_dict = spectral_lines[lines_label]
+    lines_dict = spectral_lines[lines_label]
 
-    sampler_pams = line_iter_dict['sampler_parameters']
+    norm_dict = lines_dict.get('normalization', {})
+    norm_pams = {}
+    norm_pams['normalize_transmission'] = norm_dict.get('normalize_transmission', True)
+    norm_pams['model_poly_degree'] = norm_dict.get('model_poly_degree', 2)
+    norm_pams['spectra_poly_degree'] = norm_dict.get('spectra_poly_degree', 2)
+    norm_pams['lower_threshold'] = norm_dict.get('lower_threshold', 0.950)
+    norm_pams['percentile_selection'] = norm_dict.get('percentile_selection', 10)
+
+
+    sampler_pams = lines_dict['sampler_parameters']
     sampler_name = sampler_pams.get('sampler_name', 'emcee')
 
     # TODO reference as input parameter
@@ -76,10 +92,10 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
 
     model_case = 10
 
-    fit_pams = line_iter_dict['fit_parameters']
+    fit_pams = lines_dict['fit_parameters']
     # Added compativlity to "wrong" keys
 
-    clv_rm_correction = line_iter_dict.get('clv_rm_correction', True)
+    clv_rm_correction = lines_dict.get('clv_rm_correction', True)
 
     free_Rp = fit_pams.get('free_Rp', True) \
         and fit_pams.get('free_planet_radius', True) \
@@ -94,7 +110,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
     prior_dict = fit_pams.get('priors', {}) \
         or fit_pams.get('priors', {})
 
-    if len(line_iter_dict['lines']) < 2:
+    if len(lines_dict['lines']) < 2:
         if free_Rp is True and free_winds is True:
             model_case = 0
         if free_Rp is True and free_winds is False:
@@ -143,7 +159,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
     print('      shared_winds:    (default: False) ', shared_winds)
     print('      shared_FWHM:     (default: False) ', shared_FWHM)
     print('      jitter:          (default: True)  ', jitter_flag)
-    print('      # lines:        ', len(line_iter_dict['lines']))
+    print('      # lines:        ', len(lines_dict['lines']))
     print('      model_case:     ', model_case)
 
     """ parameters list:
@@ -157,7 +173,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
     lines_center = np.empty(0)  # laboratory wavelength of spectral lines
     pam_index = 0  # keep track of the number of variables
 
-    for line_key, line_val in line_iter_dict['lines'].items():
+    for line_key, line_val in lines_dict['lines'].items():
         pam_name = line_key + '_contrast'
         pams_dict[pam_name] = pam_index
         pams_list.append(pam_name)
@@ -172,7 +188,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
         #
 
         if model_case in [0, 1, 2, 3, 10, 11, 14, 20, 21, 24]:
-            # if not line_iter_dict['fit_parameters']['shared_fwhm']:
+            # if not lines_dict['fit_parameters']['shared_fwhm']:
             pam_name = line_key + '_fwhm'
             pams_dict[pam_name] = pam_index
             pams_list.append(pam_name)
@@ -180,8 +196,8 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
             theta_start = np.append(theta_start, 5.0)
             pam_index += 1
 
-        # if line_iter_dict['fit_parameters']['fixed_separation']: continue
-        # if not line_iter_dict['fit_parameters']['lines_shift']: continue
+        # if lines_dict['fit_parameters']['fixed_separation']: continue
+        # if not lines_dict['fit_parameters']['lines_shift']: continue
 
         if model_case in [0, 2, 10, 12, 20, 22]:
             pam_name = line_key + '_winds'
@@ -192,7 +208,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
             pam_index += 1
 
     if model_case in [12, 13, 15, 22, 23, 25]:
-        # if line_iter_dict['fit_parameters']['shared_fwhm']:
+        # if lines_dict['fit_parameters']['shared_fwhm']:
         pam_name = 'shared_fwhm'
         pams_dict[pam_name] = pam_index
         pams_list.append(pam_name)
@@ -201,7 +217,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
         pam_index += 1
 
     if model_case in [11, 13, 21, 23]:
-        # if line_iter_dict['fit_parameters']['fixed_separation'] and line_iter_dict['fit_parameters']['lines_shift']:
+        # if lines_dict['fit_parameters']['fixed_separation'] and lines_dict['fit_parameters']['lines_shift']:
         pam_name = 'shared_winds'
         pams_dict[pam_name] = pam_index
         pams_list.append(pam_name)
@@ -252,8 +268,23 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
         observational_pams = load_from_cpickle(
             'observational_pams', config_in['output'], night)
 
+        # Moved here to check wheter PCA or master-out have been employed
+        preparation_input = load_from_cpickle(
+            'transmission_preparation', config_in['output'], night)
+
+        if preparation_input.get('pca_output', False):
+            if pca_iteration >= 0:
+                it_string = str(pca_iteration).zfill(2)
+            else:
+                it_string = str(preparation_input.get('ref_iteration', 0)).zfill(2)
+            preparation = preparation_input[it_string]
+        else:
+            preparation = preparation_input
+            it_string = ''
+
+
         try:
-            mcmc_data = load_from_cpickle(subroutine_name + '_data', config_in['output'], night, lines_label)
+            mcmc_data = load_from_cpickle(subroutine_name + '_data', config_in['output'], night, lines_label, it_string)
 
             clv_rm_radius = mcmc_data['clv_rm_radius']
             clv_rm_grid = mcmc_data['clv_rm_grid']
@@ -277,8 +308,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
                 'calibration_fibA', config_in['output'], night)
             input_data = retrieve_observations(
                 config_in['output'], night, lists['observations'])
-            preparation = load_from_cpickle(
-                'transmission_preparation', config_in['output'], night)
+
 
             if clv_rm_correction:
                 try:
@@ -314,7 +344,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
             print('STEP SHAPE:', np.shape(step_SRF))
 
             processed['common'] = {
-                'range': line_iter_dict['fit_parameters']['range'],
+                'range': lines_dict['fit_parameters']['range'],
                 'reference_wave': wave_SRF,
                 'reference_step': step_SRF
             }
@@ -342,7 +372,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
             print(processed['common']['size'])
 
             processed['common_extended'] = {
-                'range': line_iter_dict['range'],
+                'range': lines_dict['range'],
                 'reference_wave': wave_SRF,
                 'reference_step': step_SRF
             }
@@ -379,7 +409,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
                 1) exclusion of regions with planetary lines, taking into
                    account the planetary RV semi-amplitude
             """
-            for line_key, line_val in line_iter_dict['lines'].items():
+            for line_key, line_val in lines_dict['lines'].items():
                 line_extension = 1.2 * \
                     planet_dict['RV_semiamplitude'][0] * \
                         line_val / speed_of_light_km
@@ -471,9 +501,22 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
                 processed[obs]['continuum'] = processed['common']['reference_wave'] * 0.
                 processed[obs]['normalized'] = processed['common']['reference_wave'] * 0.
                 processed[obs]['normalized_err'] = processed['common']['reference_wave'] * 0.
-                """ We perform the selection only on the order where we actually
-                    were we actually have data for the MCMC
+                """ We perform the selection only on the order
+                    where we actually have data for the MCMC
                 """
+
+
+                """ TODO 
+                    Something seems wrang here, ratio spectra are not used...
+                """
+
+
+
+
+
+
+
+
                 for order in processed['common']['order_list']:
 
                     selection = processed['common_extended']['line_exclusion'][order, :] & (
@@ -598,7 +641,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
             }
 
             save_to_cpickle(subroutine_name + '_data', mcmc_data,
-                            config_in['output'], night, lines_label)
+                            config_in['output'], night, lines_label, it_string)
 
             # Forcing memory deallocation
             clv_rm_models = None
@@ -608,7 +651,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
         print("transmission_mcmc           ")
 
         try:
-            results_dict = load_from_cpickle(subroutine_name+'_'+sampler_name+'_results', config_in['output'], night, lines_label)
+            results_dict = load_from_cpickle(subroutine_name+'_'+sampler_name+'_results', config_in['output'], night, lines_label, it_string)
             print("   Transmission MCMC analysis for lines {0:s}, night: {1:s}  already performed".format(
                 lines_label, night))
 
@@ -642,7 +685,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
 
         # getting fit parameters
         lines_center, pams_dict, pams_list, boundaries, theta_start = define_theta_array(
-            model_case, line_iter_dict, planet_dict, n_jitter)
+            model_case, lines_dict, planet_dict, n_jitter)
         ndim = len(theta_start)
         ngen = sampler_pams.get('n_gen', 64000)
         nwalkers_mult = sampler_pams.get('n_walkers_mult', 2)
@@ -797,7 +840,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
 
 
         save_to_cpickle(subroutine_name+'_'+sampler_name+'_results',
-                        results_dict, config_in['output'], night, lines_label)
+                        results_dict, config_in['output'], night, lines_label, it_string)
 
         # TODO improve output
         print('   *** sampler output ')
@@ -824,7 +867,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
 
     print()
     try:
-        all_mcmc_data = load_from_cpickle(subroutine_name+'_data', config_in['output'], night='', lines=lines_label)
+        all_mcmc_data = load_from_cpickle(subroutine_name+'_data', config_in['output'], night='', lines=lines_label, it_string=it_string)
 
         all_clv_rm_radius = all_mcmc_data['clv_rm_radius']
         all_clv_rm_grid = all_mcmc_data['clv_rm_grid']
@@ -845,7 +888,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
 
         for night in night_dict:
 
-            mcmc_data = load_from_cpickle(subroutine_name+'_data', config_in['output'], night, lines_label)
+            mcmc_data = load_from_cpickle(subroutine_name+'_data', config_in['output'], night, lines_label, it_string=it_string)
 
             try:
                 # Building the arrays for the full analysis
@@ -907,11 +950,11 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
         }
 
         save_to_cpickle(subroutine_name+'_data', all_mcmc_data,
-                        config_in['output'], night='', lines=lines_label)
+                        config_in['output'], night='', lines=lines_label, it_string=it_string)
 
     try:
         results_dict = load_from_cpickle(subroutine_name+ '_'+ sampler_name+'_results',
-                                         config_in['output'], night='', lines=lines_label)
+                                         config_in['output'], night='', lines=lines_label, it_string=it_string)
         print("   Transmission MCMC analysis for lines {0:s} already performed ".format(
                 lines_label))
 
@@ -934,7 +977,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
     except FileNotFoundError:
 
         lines_center, pams_dict, pams_list, boundaries, theta_start = define_theta_array(
-            model_case, line_iter_dict, planet_dict, n_jitter)
+            model_case, lines_dict, planet_dict, n_jitter)
         ndim = len(theta_start)
 
         if pams_dict.get('rp_factor', False):
@@ -1090,7 +1133,7 @@ def compute_transmission_mcmc(config_in, lines_label, reference='planetRF'):
                     / planet_dict['period'][0] * 2 * np.pi
 
         save_to_cpickle(subroutine_name +'_'+sampler_name+'_results',
-                        results_dict, config_in['output'], night='', lines=lines_label)
+                        results_dict, config_in['output'], night='', lines=lines_label, it_string=it_string)
 
         for key, val in pams_dict.items():
             print('{0:24s}  {1:4d}  {2:12f}   {3:12f}  {4:12f} (15-84 p) ([{5:9f}, {6:9f}])'.format(key, val,
